#include <stdio.h>                /* prototype declarations for I/O functions */
#include <LPC214x.H>              /* LPC21xx definitions                      */

#define RDLINE 16
#define RESLINE 17
#define CSLINE 18
#define CDLINE 19
#define WRLINE 20
// macros for control lines

#define CLR_RD IOCLR0 = (1 << RDLINE)
#define SET_RD IOSET0 = (1 << RDLINE)

#define CLR_RESET IOCLR0 = (1 << RESLINE)
#define SET_RESET IOSET0 = (1 << RESLINE)

#define CLR_CS IOCLR0 = (1 << CSLINE)
#define SET_CS IOSET0 = (1 << CSLINE)

#define CLR_CD IOCLR0 = (1 << CDLINE)
#define SET_CD IOSET0 = (1 << CDLINE)

#define CLR_WR IOCLR0 = (1 << WRLINE)
#define SET_WR IOSET0 = (1 << WRLINE)

#define DATAPORT IOSET0
#define CLEARDATAPORT IOCLR0=0x0080BF00;

#define SETCONTROLPORT IOSET0
#define CLEARCONTROLPORT IOCLR0=0x001F0000; 

// color definitions
#define	BLACK	  0x0000
#define	BLUE	  0x001F
#define	RED 	  0xF800
#define	GREEN   0x07E0
#define CYAN	  0x07FF
#define MAGENTA 0xF81F
#define YELLOW  0xFFE0
#define WHITE	  0xFFFF



void mcudelay(int count)
{
  int j=0,i=0;

  for(j=0;j<count;j++)
  {
    /* At 60Mhz, the below loop introduces
    delay of 1 us */
    for(i=0;i<35;i++);
  }
}

/*************************************************/
void lcddisplaydelay(unsigned int t)
{
	unsigned char t1;
	while(t--)
	for(t1=11;t1>0;t1--)
	{mcudelay(1);}
}
/*************************************************/
void init_gpio()
{
	SCS = 0x00000000;
	PINSEL0 = 0x00000005;
	PINSEL1 = 0x00000000;
	IODIR0 = 0xFFFFFFFF;
}
/*************************************************/
void show_gpio()
{
	int i;
	for (i=0;i<2;i++)
	{
		IOCLR0 = 0XFFFFFFFF;
		IOSET0 = 0X000000FF;
		mcudelay(1000000);
		IOCLR0 = 0XFFFFFFFF;
		mcudelay(1000000);

	}
}

void write_command(unsigned int command)
{
	CLEARDATAPORT;

	CLR_CS;
	CLR_CD;
//	SET_RD;
//	SET_WR;
	DATAPORT=((0x0000BF00&(command))|(0x00800000&(command<<9)));
	CLR_WR;
	mcudelay(1);
//	CLEARDATAPORT;
	SET_WR;
	CLEARDATAPORT;
	DATAPORT=((0x0000BF00&(command<<8))|(0x00800000&(command<<17)));
	CLR_WR;
	mcudelay(1);
	SET_WR;

	SET_CS;
}

/*************************************************/
void write_data(unsigned int data)
{
	CLEARDATAPORT;

	CLR_CS;
	SET_CD;
//	SET_RD;
//	SET_WR;
	DATAPORT=(0x0000BF00&(data))|(0x00800000&(data<<9));
	CLR_WR;
	mcudelay(1);
	SET_WR;
	CLEARDATAPORT;
	DATAPORT=(0x0000BF00&(data<<8))|(0x00800000&(data<<17));
	CLR_WR;
	mcudelay(1);
	SET_WR;

	SET_CS;
}

/*************************************************/
void initialization()
{
	SET_RD;
	SET_WR;
	SET_CS;
	SET_CD;
	CLEARDATAPORT;	 
	CLR_RESET;
	lcddisplaydelay(200);
	SET_RESET;
	lcddisplaydelay(500);

	write_command(0x0028);    // VCOM OTP
	write_data(0x0006);       // Page 55-56 of SSD2119 datasheet

	write_command(0x0000);    // start Oscillator
	write_data(0x0001);       // Page 36 of SSD2119 datasheet

	write_command(0x0010);    // Sleep mode
	write_data(0x0000);       // Page 49 of SSD2119 datasheet

	write_command(0x0001);    // Driver Output Control
	write_data(0x32EF);       // Page 36-39 of SSD2119 datasheet

	write_command(0x0002);    // LCD Driving Waveform Control
	write_data(0x0600);       // Page 40-42 of SSD2119 datasheet

	write_command(0x0003);    // Power Control 1
	write_data(0x6A38);       // Page 43-44 of SSD2119 datasheet

	write_command(0x0011);    // Entry Mode
	write_data(0x6870);       // Page 50-52 of SSD2119 datasheet

	write_command(0X000F);    // Gate Scan Position
	write_data(0x0000);       // Page 49 of SSD2119 datasheet

	write_command(0X000B);    // Frame Cycle Control
	write_data(0x5308);       // Page 45 of SSD2119 datasheet

	write_command(0x000C);    // Power Control 2
	write_data(0x0003);       // Page 47 of SSD2119 datasheet

	write_command(0x000D);    // Power Control 3
	write_data(0x000A);       // Page 48 of SSD2119 datasheet

	write_command(0x000E);    // Power Control 4
	write_data(0x2E00);       // Page 48 of SSD2119 datasheet

	write_command(0x001E);    // Power Control 5
	write_data(0x00BE);       // Page 53 of SSD2119 datasheet

	write_command(0x0025);    // Frame Frequency Control
	write_data(0x8000);       // Page 53 of SSD2119 datasheet

	write_command(0x0026);    // Analog setting
	write_data(0x7800);       // Page 54 of SSD2119 datasheet

	write_command(0x004E);    // Ram Address Set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet

	write_command(0x004F);    // Ram Address Set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet

	write_command(0x0012);    // Sleep mode
	write_data(0x08D9);       // Page 49 of SSD2119 datasheet

	// Gamma Control (R30h to R3Bh) -- Page 56 of SSD2119 datasheet
	write_command(0x0030);
	write_data(0x0000);

	write_command(0x0031);
	write_data(0x0104);

	write_command(0x0032);
	write_data(0x0100);

	write_command(0x0033);
	write_data(0x0305);

	write_command(0x0034);
	write_data(0x0505);

	write_command(0x0035);
	write_data(0x0305);

	write_command(0x0036);
	write_data(0x0707);

	write_command(0x0037);
	write_data(0x0300);

	write_command(0x003A);
	write_data(0x1200);

	write_command(0x003B);
	write_data(0x0800);		 

	write_command(0x0007);    // Display Control 
	write_data(0x0033);       // Page 45 of SSD2119 datasheet

	lcddisplaydelay(1000);

	write_command(0x0022);    // RAM data write/read
}
/*************************************************/
void Display_Home()
{
	write_command(0x004E);    // RAM address set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet
	write_command(0x004F);    // RAM address set
	write_data(0x0000);       // Page 58 of SSD2119 datasheet

	write_command(0x0044);    // Vertical RAM address position
	write_data(0xEF00);       // Page 57 of SSD2119 datasheet
	write_command(0x0045);    // Horizontal RAM address position 
	write_data(0x0000);       // Page 57 of SSD2119 datasheet
	write_command(0x0046);    // Horizontal RAM address position
	write_data(0x013F);       // Page 57 of SSD2119 datasheet

	write_command(0x0022);    // RAM data write/read
}


void display_rgb(unsigned int data)
{
	unsigned int i,j;
	Display_Home();
	for(i=0;i<320;i++)
  {
  	for(j=0;j<240;j++)
  	{
  		write_data(data);
  	}
  }
}
/*************************************************/
void LCD_test()
{
	unsigned int i,j;
	Display_Home();

	for(i=0;i<320;i++)
	{
		for(j=0;j<240;j++)
		{
			if(i>279)write_data(BLACK);
			else if(i>239)write_data(BLUE);
			else if(i>199)write_data(GREEN);
			else if(i>159)write_data(CYAN);
			else if(i>119)write_data(RED);
			else if(i>79)write_data(MAGENTA);
			else if(i>39)write_data(YELLOW);
			else write_data(WHITE);
		}
	}
}

/*************************************************/
void init_displaycontroller()
{
	lcddisplaydelay(0xff);
	initialization();
	LCD_test();
	lcddisplaydelay(0xff);      
}
/*************************************************/

int main (void)  
{
	int i;  
	init_gpio();
	init_displaycontroller();

//	for (i=0;i<64000;i++) 
//	{
//		display_rgb(RED);
//		mcudelay(5000000);                          /* An embedded program does not stop */
//	}                               
}
